@@ -1,4 +1,4 @@
-get_predictions_geeglm <- function(model, fitfram, ci.lvl, type, model_class, value_adjustment, terms, condition, ...) {
+get_predictions_geeglm <- function(model, fitfram, ci.lvl, linv, type, model_class, value_adjustment, terms, condition, ...) {
   se <- (!is.null(ci.lvl) && !is.na(ci.lvl))
 
   # compute ci, two-ways
@@ -36,13 +36,13 @@ get_predictions_geeglm <- function(model, fitfram, ci.lvl, type, model_class, va
     )
 
 
-  if (!is.null(se.pred) && isTRUE(se)) {
+  if (.check_returned_se(se.pred) && isTRUE(se)) {
     se.fit <- se.pred$se.fit
     fitfram <- se.pred$prediction_data
 
     # CI
-    fitfram$conf.low <- fitfram$predicted - stats::qnorm(ci) * se.fit
-    fitfram$conf.high <- fitfram$predicted + stats::qnorm(ci) * se.fit
+    fitfram$conf.low <- linv(fitfram$predicted - stats::qnorm(ci) * se.fit)
+    fitfram$conf.high <- linv(fitfram$predicted + stats::qnorm(ci) * se.fit)
 
     # copy standard errors
     attr(fitfram, "std.error") <- se.fit
@@ -53,5 +53,7 @@ get_predictions_geeglm <- function(model, fitfram, ci.lvl, type, model_class, va
     fitfram$conf.high <- NA
   }
 
+  fitfram$predicted <- linv(fitfram$predicted)
+
   fitfram
 }
