@@ -1,8 +1,4 @@
 #' @rdname ggpredict
-#'
-#' @importFrom stats na.omit
-#' @importFrom sjlabelled as_numeric
-#' @importFrom insight find_predictors link_inverse print_color
 #' @export
 ggeffect <- function(model, terms, ci.lvl = .95, ...) {
 
@@ -18,6 +14,11 @@ ggeffect <- function(model, terms, ci.lvl = .95, ...) {
     terms <- all.vars(terms)
   }
 
+  # tidymodels?
+  if (inherits(model, "model_fit")) {
+    model <- model$fit
+  }
+
   if (inherits(model, "list")  && !inherits(model, c("bamlss", "maxLik"))) {
     res <- lapply(model, function(.x) ggeffect_helper(.x, terms, ci.lvl, ...))
   } else {
@@ -66,7 +67,7 @@ ggeffect_helper <- function(model, terms, ci.lvl, ...) {
   # check whether we have an argument "transformation" for effects()-function
   # in this case, we need another default title, since we have
   # non-transformed effects
-  additional_dot_args <- lapply(match.call(expand.dots = F)$`...`, function(x) x)
+  additional_dot_args <- lapply(match.call(expand.dots = FALSE)$`...`, function(x) x)
   # check whether we have a "transformation" argument
   t.add <- which(names(additional_dot_args) == "transformation")
   # if we have a "transformation" argument, and it's NULL,
