@@ -18,6 +18,19 @@
 #'   \code{facets}.
 #' @param add.data,rawdata Logical, if \code{TRUE}, a layer with raw data from response by
 #'   predictor on the x-axis, plotted as point-geoms, is added to the plot.
+#' @param limit.range Logical, if \code{TRUE}, limits the range of the prediction
+#'   bands to the range of the data.
+#' @param residuals Logical, if \code{TRUE}, a layer with partial residuals is
+#'   added to the plot. See vignette \href{https://cran.r-project.org/package=effects}{"Effect Displays with Partial Residuals"}
+#'   from \pkg{effects} for more details on partial residual plots.
+#' @param residuals.line Logical, if \code{TRUE}, a loess-fit line is added to the
+#'   partial residuals plot. Only applies if \code{residuals} is \code{TRUE}.
+#' @param collapse.group For mixed effects models, name of the grouping variable
+#'   of random effects. If \code{collapse.group = TRUE}, data points "collapsed"
+#'   by the first random effect groups are added to the plot. Else, if
+#'   \code{collapse.group} is a name of a group factor, data is collapsed by
+#'   that specific random effect. See \code{\link{collapse_by_group}} for
+#'   further details.
 #' @param colors Character vector with color values in hex-format, valid
 #'   color value names (see \code{demo("colors")}) or a name of a
 #'   ggeffects-color-palette.
@@ -63,9 +76,12 @@
 #' @param base_family Base font family.
 #' @param ... Further arguments passed down to \code{ggplot::scale_y*()}, to
 #'    control the appearance of the y-axis.
+#' @param residuals.type Deprecated. Formally was the residual type. Now is always \code{"working"}.
 #'
 #' @inheritParams get_title
 #'
+#' @inheritSection residualize_over_grid Partial Residuals
+#'
 #' @return A ggplot2-object.
 #'
 #' @note Load \code{library(ggplot2)} and use \code{theme_set(theme_ggeffects())}
@@ -76,7 +92,7 @@
 #'   There are pre-defined colour palettes in this package. Use
 #'   \code{show_pals()} to show all available colour palettes.
 #'
-#' @details For proportional odds logistic regression (see \code{\link[MASS]{polr}})
+#' @details For proportional odds logistic regression (see \code{?MASS::polr})
 #'   or cumulative link models in general, plots are automatically facetted
 #'   by \code{response.level}, which indicates the grouping of predictions
 #'   based on the level of the model's response.
@@ -109,15 +125,16 @@
 #'
 #' # show all color palettes
 #' show_pals()
-#'
-#' @importFrom stats binomial poisson gaussian Gamma inverse.gaussian quasi quasibinomial quasipoisson
-#' @importFrom sjlabelled as_numeric
 #' @export
 plot.ggeffects <- function(x,
                            ci = TRUE,
                            ci.style = c("ribbon", "errorbar", "dash", "dot"),
                            facets,
                            add.data = FALSE,
+                           limit.range = FALSE,
+                           residuals = FALSE,
+                           residuals.line = FALSE,
+                           collapse.group = FALSE,
                            colors = "Set1",
                            alpha = .15,
                            dodge = .25,
@@ -136,12 +153,15 @@ plot.ggeffects <- function(x,
                            grid,
                            one.plot = TRUE,
                            rawdata,
+                           residuals.type,
                            ...) {
 
   if (!requireNamespace("ggplot2", quietly = FALSE)) {
     stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
   }
 
+  if (!missing(residuals.type)) warning("'residuals.type' is deprecated. Using 'working' residuals.")
+
   # check alias
   if (missing(rawdata)) rawdata <- add.data
 
@@ -173,7 +193,7 @@ plot.ggeffects <- function(x,
   if (missing(ci.style) && x_is_factor) ci.style <- "errorbar"
   ci.style <- match.arg(ci.style)
 
-  add.args <- lapply(match.call(expand.dots = F)$`...`, function(x) x)
+  add.args <- lapply(match.call(expand.dots = FALSE)$`...`, function(x) x)
   if (!("breaks" %in% names(add.args)) && isTRUE(log.y)) {
     y.breaks <- unique(round(log2(pretty(c(min(x$conf.low), max(x$conf.high))))))
     y.breaks[is.nan(y.breaks)] <- NA
@@ -197,11 +217,87 @@ plot.ggeffects <- function(x,
   has_facets <- .obj_has_name(x, "facet") && length(unique(x$facet)) > 1
   has_panel <- .obj_has_name(x, "panel") && length(unique(x$panel)) > 1
 
+
+  # if we add data points, limit to range
+  if (isTRUE(limit.range)) {
+    raw_data <- attr(x, "rawdata", exact = TRUE)
+    if (!is.null(raw_data)) {
+      if (has_groups && has_facets) {
+        ranges <- lapply(split(raw_data, list(raw_data$group, raw_data$facet)), function(i) range(i$x, na.rm = TRUE))
+        for (i in unique(raw_data$group)) {
+          for (j in unique(raw_data$facet)) {
+            if (any(is.infinite(ranges[[paste0(i, ".", j)]]))) {
+              remove <- x$group == i & x$facet == j
+              x$x[remove] <- NA
+            } else {
+              remove <- x$group == i & x$facet == j & x$x < ranges[[paste0(i, ".", j)]][1]
+              x$x[remove] <- NA
+              remove <- x$group == i & x$facet == j & x$x > ranges[[paste0(i, ".", j)]][2]
+              x$x[remove] <- NA
+            }
+          }
+        }
+      } else if (has_groups) {
+        ranges <- lapply(split(raw_data, raw_data$group), function(i) range(i$x, na.rm = TRUE))
+        for (i in names(ranges)) {
+          remove <- x$group == i & x$x < ranges[[i]][1]
+          x$x[remove] <- NA
+          remove <- x$group == i & x$x > ranges[[i]][2]
+          x$x[remove] <- NA
+        }
+      } else {
+        remove <- x$x < min(raw_data$x, na.rm = TRUE) | x$x > max(raw_data$x, na.rm = TRUE)
+        x$x[remove] <- NA
+      }
+    }
+  }
+
+
+  # partial residuals?
+  if (residuals) {
+    model <- .get_model_object(x)
+    if (!is.null(model)) {
+      residual_data <- residualize_over_grid(grid = x, model = model)
+      attr(x, "residual_data") <- residual_data
+
+      ## TODO for now, we allow no continuous grouping varialbles for partial residuals
+      # it is difficult to match "raw data" values with the specific at-values
+      # for continuous variables
+
+      attr(x, "continuous.group") <- FALSE
+    } else {
+      warning("Could not find model object to extract residuals.", call. = FALSE)
+      residals <- FALSE
+    }
+  }
+
+
+  # collapse data by random effects?
+  if (isTRUE(collapse.group) || (!is.null(collapse.group) && !isFALSE(collapse.group))) {
+    if (isTRUE(collapse.group)) {
+      # use first random effect
+      collapse.group <- NULL
+    }
+    re_data <- collapse_by_group(x,
+                                 model = .get_model_object(x),
+                                 collapse.by = collapse.group,
+                                 residuals = residuals)
+    attr(x, "random_effects_data") <- re_data
+    attr(x, "continuous.group") <- FALSE
+
+    # no additional residuals or raw data
+    rawdata <- add.data <- FALSE
+    residuals <- FALSE
+    attr(x, "residual_data") <- NULL
+  }
+
+
   # convert x back to numeric
   if (!is.numeric(x$x)) {
-    if (x_is_factor && .is_numeric_factor(x$x))
+    if (x_is_factor && !.is_numeric_factor(x$x)) {
       levels(x$x) <- seq_len(nlevels(x$x))
-    x$x <- sjlabelled::as_numeric(x$x)
+    }
+    x$x <- .factor_to_numeric(x$x)
   }
 
   # special solution for polr
@@ -236,7 +332,7 @@ plot.ggeffects <- function(x,
       x$facet <- sprintf(
         "%s = %g",
         attr(x, "terms", exact = TRUE)[3],
-        sjlabelled::as_numeric(x$facet)
+        .factor_to_numeric(x$facet)
       )
     }
   }
@@ -268,7 +364,7 @@ plot.ggeffects <- function(x,
       }
 
       pl <- plot_panel(
-        x = x[x$panel == .p, ],
+        x = x[x$panel == .p, , drop = FALSE],
         colors = colors,
         has_groups = has_groups,
         facets_grp = facets_grp,
@@ -288,6 +384,8 @@ plot.ggeffects <- function(x,
         jitter = jitter,
         jitter.miss = jitter.miss,
         rawdata = rawdata,
+        residuals = residuals,
+        residuals.line = residuals.line,
         show.title = show.title,
         show.x.title = show.x.title,
         show.y.title = show.y.title,
@@ -334,6 +432,8 @@ plot.ggeffects <- function(x,
       jitter = jitter,
       jitter.miss = jitter.miss,
       rawdata = rawdata,
+      residuals = residuals,
+      residuals.line = residuals.line,
       show.title = show.title,
       show.x.title = show.x.title,
       show.y.title = show.y.title,
@@ -355,7 +455,6 @@ plot.ggeffects <- function(x,
 }
 
 
-
 plot_panel <- function(x,
                        colors,
                        has_groups,
@@ -376,6 +475,8 @@ plot_panel <- function(x,
                        jitter,
                        jitter.miss,
                        rawdata,
+                       residuals,
+                       residuals.line,
                        show.title,
                        show.x.title,
                        show.y.title,
@@ -396,21 +497,52 @@ plot_panel <- function(x,
     x$group_col <- x$group
   }
 
-  # base plot, set mappings
+  # base plot, set mappings -----
+
+  plot_data <- x[!is.na(x$x), ]
+
   if (has_groups && !facets_grp && is_black_white && x_is_factor)
-    p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", shape = "group"))
+    p <- ggplot2::ggplot(plot_data, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", shape = "group"))
   else if (has_groups && !facets_grp && is_black_white && !x_is_factor)
-    p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", linetype = "group"))
+    p <- ggplot2::ggplot(plot_data, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", linetype = "group"))
   else if (has_groups && !facets_grp && colors[1] == "gs" && x_is_factor)
-    p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", shape = "group"))
+    p <- ggplot2::ggplot(plot_data, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", shape = "group"))
   else if (has_groups && colors[1] != "bw")
-    p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col"))
+    p <- ggplot2::ggplot(plot_data, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col"))
   else
-    p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted"))
+    p <- ggplot2::ggplot(plot_data, ggplot2::aes_string(x = "x", y = "predicted"))
 
 
-  # get color values
-  colors <- .get_colors(colors, length(unique(x$group)), isTRUE(attr(x, "continuous.group")))
+  # get color values -----
+
+  colors <- .get_colors(colors, length(unique(stats::na.omit(x$group))), isTRUE(attr(x, "continuous.group")))
+
+
+  # plot raw data points -----
+
+  # get raw data
+  rawdat <- attr(x, "rawdata", exact = TRUE)
+  if (rawdata) {
+    p <- .add_raw_data_to_plot(p, x, rawdat, ci.style, dot.alpha, dot.size, dodge, jitter, jitter.miss, colors)
+  }
+
+
+  # plot partial residuals -----
+
+  # get residual data
+  residual_data <- attr(x, "residual_data", exact = TRUE)
+  if (isTRUE(residuals)) {
+    p <- .add_residuals_to_plot(p, x, residual_data, residuals.line, ci.style, line.size, dot.alpha, dot.size, dodge, jitter, colors)
+  }
+
+
+  # plot random effects group data -----
+
+  # get re-group data
+  random_effects_data <- attr(x, "random_effects_data", exact = TRUE)
+  if (!is.null(random_effects_data)) {
+    p <- .add_re_data_to_plot(p, x, random_effects_data, dot.alpha, dot.size, dodge, jitter)
+  }
 
 
   # now plot the geom. we use a smoother for a continuous x, and
@@ -438,7 +570,8 @@ plot_panel <- function(x,
   }
 
 
-  # CI?
+  # CI ----
+
   if (ci) {
 
     # for a factor on x-axis, use error bars
@@ -515,11 +648,12 @@ plot_panel <- function(x,
   x_lab <- get_x_labels(x, case)
 
   if (!is.null(x_lab)) {
-    p <- p + ggplot2::scale_x_continuous(breaks = unique(x$x), labels = x_lab)
+    p <- p + ggplot2::scale_x_continuous(breaks = unique(plot_data$x), labels = x_lab)
   }
 
 
-  # facets?
+  # facets ----
+
   if (facets_grp) {
     # facet groups
     p <- p + ggplot2::facet_wrap(~group, scales = "free_x")
@@ -532,97 +666,9 @@ plot_panel <- function(x,
   }
 
 
-  # plot raw data points. we need an own aes for this
-  if (rawdata) {
-    # get raw data and check, if any data available
-    rawdat <- attr(x, "rawdata", exact = TRUE)
+  # set colors ----
 
-    if (!is.null(rawdat)) {
-      # make sure response is numeric
-      rawdat$response <- sjlabelled::as_numeric(rawdat$response)
-
-      # check if we have a group-variable with at least two groups
-      if (.obj_has_name(rawdat, "group")) {
-
-        if (isTRUE(attr(x, "continuous.group"))) {
-          rawdat$group_col <- as.numeric(as.character(rawdat$group))
-        } else {
-          rawdat$group_col <- rawdat$group
-        }
-
-        rawdat$group <- as.factor(rawdat$group)
-        # levels(rawdat$group) <- unique(x$group)
-        grps <- .n_distinct(rawdat$group) > 1
-      } else {
-        grps <- FALSE
-      }
-
-      # check if we have only selected values for groups, in this case
-      # filter raw data to match grouping colours
-      if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(rawdat$group) > .n_distinct(x$group)) {
-        rawdat <- rawdat[which(rawdat$group %in% x$group), ]
-      }
-
-
-      # if we have groups, add colour aes, to map raw data to
-      # grouping variable
-
-      if (grps)
-        mp <- ggplot2::aes_string(x = "x", y = "response", colour = "group_col")
-      else
-        mp <- ggplot2::aes_string(x = "x", y = "response")
-
-
-      # for binary response, no jittering by default
-
-      if ((attr(x, "logistic", exact = TRUE) == "1" && jitter.miss) || is.null(jitter)) {
-        p <- p + ggplot2::geom_point(
-          data = rawdat,
-          mapping = mp,
-          alpha = dot.alpha,
-          size = dot.size,
-          show.legend = FALSE,
-          inherit.aes = FALSE,
-          shape = 16
-        )
-      } else {
-        if (ci.style == "errorbar") {
-          p <- p + ggplot2::geom_point(
-            data = rawdat,
-            mapping = mp,
-            alpha = dot.alpha,
-            size = dot.size,
-            position = ggplot2::position_jitterdodge(
-              jitter.width = jitter[1],
-              jitter.height = jitter[2],
-              dodge.width = dodge
-            ),
-            show.legend = FALSE,
-            inherit.aes = FALSE,
-            shape = 16
-          )
-
-        } else {
-          p <- p + ggplot2::geom_jitter(
-            data = rawdat,
-            mapping = mp,
-            alpha = dot.alpha,
-            size = dot.size,
-            width = jitter[1],
-            height = jitter[2],
-            show.legend = FALSE,
-            inherit.aes = FALSE,
-            shape = 16
-          )
-        }
-      }
-    } else {
-      message("Raw data not available.")
-    }
-  }
-
-  # set colors
-  if(isTRUE(rawdata) && isTRUE(attr(x, "continuous.group"))) {
+  if (isTRUE(rawdata) && isTRUE(attr(x, "continuous.group"))) {
     p <- p +
       ggplot2::scale_color_gradientn(colors = colors, aesthetics = c("colour", "fill"), guide = "legend", breaks = as.numeric(levels(x$group)), limits = range(c(rawdat$group_col, x$group_col)))
   } else {
@@ -631,13 +677,16 @@ plot_panel <- function(x,
   }
 
 
-  # show/hide titles
+  # show/hide titles ----
+
   if (!show.title) attr(x, "title") <- NULL
   if (!show.title) attr(x, "n.trials") <- NULL
   if (!show.x.title) attr(x, "x.title") <- NULL
   if (!show.y.title) attr(x, "y.title") <- NULL
 
-  # set axis titles
+
+  # set axis titles ----
+
   p <- p + ggplot2::labs(
     title = get_title(x, case),
     x = get_x_title(x, case),
@@ -653,7 +702,9 @@ plot_panel <- function(x,
       shape = get_legend_title(x, case)
     )
 
-  # no legend for fill-aes
+
+  # no legend for fill-aes ----
+
   p <- p + ggplot2::guides(fill = "none")
 
   if (is_black_white) {
@@ -663,7 +714,8 @@ plot_panel <- function(x,
   }
 
 
-  # show or hide legend?
+  # show or hide legend -----
+
   if (!show.legend) {
     p <- p + ggplot2::labs(
       colour = NULL,
@@ -673,27 +725,16 @@ plot_panel <- function(x,
   }
 
 
-  # for binomial family, fix coord
+  # for binomial family, fix coord ----
 
   if (attr(x, "logistic", exact = TRUE) == "1" && attr(x, "is.trial", exact = TRUE) == "0") {
-
-    if (!requireNamespace("scales", quietly = FALSE)) {
-      warning("Package `scales` needed to use percentage values for the y-axis. Install it by typing `install.packages(\"scales\", dependencies = TRUE)` into the console.", call. = FALSE)
-      if (log.y) {
-        if (is.null(y.breaks))
-          p <- p + ggplot2::scale_y_log10(...)
-        else
-          p <- p + ggplot2::scale_y_log10(breaks = y.breaks, limits = y.limits, ...)
-      } else
-        p <- p + ggplot2::scale_y_continuous(...)
+    if (log.y) {
+      if (is.null(y.breaks))
+        p <- p + ggplot2::scale_y_log10(labels = .percents, ...)
+      else
+        p <- p + ggplot2::scale_y_log10(labels = .percents, breaks = y.breaks, limits = y.limits, ...)
     } else {
-      if (log.y) {
-        if (is.null(y.breaks))
-          p <- p + ggplot2::scale_y_log10(labels = scales::percent, ...)
-        else
-          p <- p + ggplot2::scale_y_log10(labels = scales::percent, breaks = y.breaks, limits = y.limits, ...)
-      } else
-        p <- p + ggplot2::scale_y_continuous(labels = scales::percent, ...)
+      p <- p + ggplot2::scale_y_continuous(labels = .percents, ...)
     }
   } else if (log.y) {
     if (is.null(y.breaks))
@@ -716,7 +757,6 @@ plot_panel <- function(x,
 
 
 
-#' @importFrom graphics plot
 #' @export
 plot.ggalleffects <- function(x,
                               ci = TRUE,
@@ -765,11 +805,11 @@ plot.ggalleffects <- function(x,
     attr(dat, "rawdata") <- rawdat
 
     # set various attributes
-    attr(dat, "x.is.factor") <- attr(x[[1]], "x.is.factor", exact = T)
-    attr(dat, "family") <- attr(x[[1]], "family", exact = T)
-    attr(dat, "link") <- attr(x[[1]], "link", exact = T)
-    attr(dat, "logistic") <- attr(x[[1]], "logistic", exact = T)
-    attr(dat, "fitfun") <- attr(x[[1]], "fitfun", exact = T)
+    attr(dat, "x.is.factor") <- attr(x[[1]], "x.is.factor", exact = TRUE)
+    attr(dat, "family") <- attr(x[[1]], "family", exact = TRUE)
+    attr(dat, "link") <- attr(x[[1]], "link", exact = TRUE)
+    attr(dat, "logistic") <- attr(x[[1]], "logistic", exact = TRUE)
+    attr(dat, "fitfun") <- attr(x[[1]], "fitfun", exact = TRUE)
 
     graphics::plot(
       x = dat,
@@ -819,3 +859,319 @@ plot.ggalleffects <- function(x,
     })
   }
 }
+
+
+
+.percents <- function(x) {
+  insight::format_value(x = x, as_percent = TRUE, digits = 0)
+}
+
+
+
+
+
+.add_raw_data_to_plot <- function(p, x, rawdat, ci.style, dot.alpha, dot.size, dodge, jitter, jitter.miss, colors) {
+
+  if (!requireNamespace("ggplot2", quietly = FALSE)) {
+    stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
+  }
+
+  # we need an own aes for this
+  # we plot rawdata first, so it doesn't overlay the
+  # dots / lines for marginal effects
+
+  if (!is.null(rawdat)) {
+    # recode binary response to numeric? if so, make sure it starts with 0
+    if (identical(attributes(x)$logistic, "1")) {
+      lowest <- 0
+    } else {
+      lowest <- NULL
+    }
+    # make sure response is numeric
+    rawdat$response <- .factor_to_numeric(rawdat$response, lowest = lowest)
+
+    # check if we have a group-variable with at least two groups
+    if (.obj_has_name(rawdat, "group")) {
+
+      if (isTRUE(attr(x, "continuous.group"))) {
+        rawdat$group_col <- as.numeric(as.character(rawdat$group))
+      } else {
+        rawdat$group_col <- rawdat$group
+      }
+
+      rawdat$group <- as.factor(rawdat$group)
+      # levels(rawdat$group) <- unique(x$group)
+      grps <- .n_distinct(rawdat$group) > 1
+    } else {
+      grps <- FALSE
+    }
+
+    # check if we have only selected values for groups, in this case
+    # filter raw data to match grouping colours
+    if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(rawdat$group) > .n_distinct(x$group)) {
+      rawdat <- rawdat[which(rawdat$group %in% x$group), , drop = FALSE]
+    }
+
+
+    # if we have groups, add colour aes, to map raw data to
+    # grouping variable
+
+    if (grps)
+      mp <- ggplot2::aes_string(x = "x", y = "response", colour = "group_col")
+    else
+      mp <- ggplot2::aes_string(x = "x", y = "response")
+
+
+    # for binary response, no jittering by default
+
+    if ((attr(x, "logistic", exact = TRUE) == "1" && jitter.miss) || is.null(jitter)) {
+      p <- p + ggplot2::geom_point(
+        data = rawdat,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    } else {
+
+      # no jitter
+      if (is.null(jitter)) {
+        jitter <- c(0, 0)
+      }
+
+      if (ci.style == "errorbar") {
+        if (grps) {
+          p <- p + ggplot2::geom_point(
+            data = rawdat,
+            mapping = ggplot2::aes_string(x = "x", y = "response", colour = "group_col"),
+            alpha = dot.alpha,
+            size = dot.size,
+            position = ggplot2::position_jitterdodge(
+              jitter.width = jitter[1],
+              jitter.height = jitter[2],
+              dodge.width = dodge
+            ),
+            show.legend = FALSE,
+            inherit.aes = FALSE,
+            shape = 16
+          )
+        } else {
+          p <- p + ggplot2::geom_point(
+            data = rawdat,
+            mapping = ggplot2::aes_string(x = "x", y = "response", fill = "group_col"),
+            alpha = dot.alpha,
+            size = dot.size,
+            position = ggplot2::position_jitterdodge(
+              jitter.width = jitter[1],
+              jitter.height = jitter[2],
+              dodge.width = dodge
+            ),
+            show.legend = FALSE,
+            inherit.aes = FALSE,
+            shape = 16,
+            color = colors[1]
+          )
+        }
+      } else {
+        p <- p + ggplot2::geom_jitter(
+          data = rawdat,
+          mapping = mp,
+          alpha = dot.alpha,
+          size = dot.size,
+          width = jitter[1],
+          height = jitter[2],
+          show.legend = FALSE,
+          inherit.aes = FALSE,
+          shape = 16
+        )
+      }
+    }
+  } else {
+    message("Raw data not available.")
+  }
+
+  p
+}
+
+
+
+
+.add_residuals_to_plot <- function(p, x, residuals, residuals.line, ci.style, line.size, dot.alpha, dot.size, dodge, jitter, colors) {
+  if (!requireNamespace("ggplot2", quietly = FALSE)) {
+    stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
+  }
+
+  if (!is.null(residuals)) {
+
+    # make sure x on x-axis is on same scale
+    if (is.numeric(x$x) && !is.numeric(residuals$x)) {
+      residuals$x <- .factor_to_numeric(residuals$x)
+    }
+
+    residuals$facet <- NULL
+    residuals$panel <- NULL
+
+
+    # check if we have a group-variable with at least two groups
+    if (.obj_has_name(residuals, "group")) {
+
+      if (isTRUE(attr(x, "continuous.group")) && is.numeric(x$group)) {
+        residuals$group_col <- as.numeric(as.character(residuals$group))
+      } else {
+        residuals$group_col <- as.factor(residuals$group)
+      }
+
+      residuals$group <- as.factor(residuals$group)
+      grps <- .n_distinct(residuals$group) > 1
+    } else {
+      grps <- FALSE
+    }
+
+    # check if we have only selected values for groups, in this case
+    # filter raw data to match grouping colours
+    if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(residuals$group) > .n_distinct(x$group)) {
+      residuals <- residuals[which(residuals$group %in% x$group), , drop = FALSE]
+    }
+
+
+    # if we have groups, add colour aes, to map raw data to
+    # grouping variable
+
+    if (grps)
+      mp <- ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col")
+    else
+      mp <- ggplot2::aes_string(x = "x", y = "predicted")
+
+
+
+    # if ("group" %in% colnames(residuals)) {
+    #   if (isTRUE(attr(x, "continuous.group"))) {
+    #     residuals$group_col <- as.numeric(as.character(residuals$group))
+    #   } else {
+    #     residuals$group_col <- residuals$group
+    #   }
+    #   residuals$group <- as.factor(residuals$group)
+    #   mp <- ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col")
+    # } else {
+    #   mp <- ggplot2::aes_string(x = "x", y = "predicted")
+    # }
+
+
+
+    if (is.null(jitter)) {
+      p <- p + ggplot2::geom_point(
+        data = residuals,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    } else {
+      p <- p + ggplot2::geom_jitter(
+        data = residuals,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        width = jitter[1],
+        height = jitter[2],
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    }
+
+    if (isTRUE(residuals.line)) {
+      p <- p + ggplot2::geom_smooth(
+        data = residuals,
+        mapping = mp,
+        method = "loess",
+        inherit.aes = FALSE,
+        size = line.size,
+        se = FALSE
+      )
+    }
+
+  } else {
+    message("Partial residuals not available.")
+  }
+
+  p
+}
+
+
+
+
+
+.add_re_data_to_plot <- function(p, x, random_effects_data, dot.alpha, dot.size, dodge, jitter) {
+  if (!requireNamespace("ggplot2", quietly = FALSE)) {
+    stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
+  }
+
+  # make sure x on x-axis is on same scale
+  if (is.numeric(x$x) && !is.numeric(random_effects_data$x)) {
+    random_effects_data$x <- .factor_to_numeric(random_effects_data$x)
+  }
+
+  if ("response" %in% names(random_effects_data)) {
+    mp <- ggplot2::aes_string(x = "x", y = "response", colour = "group_col")
+  } else {
+    mp <- ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col")
+  }
+
+  if (is.null(jitter)) {
+    p <- p + ggplot2::geom_point(
+      data = random_effects_data,
+      mapping = mp,
+      alpha = dot.alpha,
+      size = dot.size,
+      position = ggplot2::position_dodge(width = dodge),
+      show.legend = FALSE,
+      inherit.aes = FALSE,
+      shape = 16
+    )
+  } else {
+    p <- p + ggplot2::geom_point(
+      data = random_effects_data,
+      mapping = mp,
+      alpha = dot.alpha,
+      size = dot.size,
+      position = ggplot2::position_jitterdodge(
+        jitter.width = jitter[1],
+        jitter.height = jitter[2],
+        dodge.width = dodge
+      ),
+      show.legend = FALSE,
+      inherit.aes = FALSE,
+      shape = 16
+    )
+  }
+
+  p
+}
+
+
+
+
+.get_model_object <- function(x) {
+  obj_name <- attr(x, "model.name", exact = TRUE)
+  model <- NULL
+  if (!is.null(obj_name)) {
+    model <- tryCatch({
+      get(obj_name, envir = parent.frame())
+    }, error = function(e) {
+      NULL
+    })
+    if (is.null(model)) {
+      model <- tryCatch({
+        get(obj_name, envir = globalenv())
+      }, error = function(e) {
+        NULL
+      })
+    }
+  }
+  model
+}
