@@ -1,11 +1,18 @@
-#' @importFrom utils packageVersion
-#' @importFrom stats terms median
-#' @importFrom sjlabelled as_numeric
-#' @importFrom insight find_predictors find_response find_random find_weights get_weights
-# factor_adjustment indicates if factors should be held constant or not
+# "factor_adjustment" indicates if factors should be held constant or not
 # need to be false for computing std.error for merMod objects
-.data_grid <- function(model, model_frame, terms, value_adjustment, factor_adjustment = TRUE, show_pretty_message = TRUE, condition = NULL, emmeans.only = FALSE) {
-  # special handling for coxph
+
+# value_adjustment is the function to calculate at which value non-focal
+# terms are held constant (mean, median, ...)
+
+.data_grid <- function(model,
+                       model_frame,
+                       terms,
+                       value_adjustment,
+                       factor_adjustment = TRUE,
+                       show_pretty_message = TRUE,
+                       condition = NULL,
+                       emmeans.only = FALSE) {
+    # special handling for coxph
   if (inherits(model, c("coxph", "coxme"))) {
     surv.var <- which(colnames(model_frame) == insight::find_response(model))
     model_frame <- .remove_column(model_frame, surv.var)
@@ -42,32 +49,29 @@
   all_terms <- .clean_terms(terms)
 
 
-  # check if user has any predictors with log-transformatio inside
+  # check if user has any predictors with log-transformation inside
   # model formula, but *not* used back-transformation "exp". Tell user
   # so she's aware of the problem
 
-  tryCatch(
+  offset_log_term <- tryCatch(
     {
+      olt <- NULL
       if (!inherits(model, "brmsfit") && show_pretty_message && .has_log(model)) {
-        check1 <- .get_offset_log_terms(model)
-        check2 <- .get_log_terms(model)
-
+        any_offset_log_term <- .get_offset_log_terms(model)
         # check if we have offset() in formula, with transformed variable
-        if (any(check1)) {
+        if (any(any_offset_log_term)) {
           clean.term <- insight::find_predictors(model, effects = "all", component = "all", flatten = FALSE)
-          clean.term <- unlist(clean.term[c("conditional", "random", "instruments")])[check1]
+          clean.term <- unlist(clean.term[c("conditional", "random", "instruments")])[any_offset_log_term]
 
           # try to back-transform
           offset_function <- .get_offset_transformation(model)
           if (identical(offset_function, "log")) {
-            model_frame[[clean.term]] <- exp(model_frame[[clean.term]])
-          } else {
-            insight::print_color(sprintf("Model uses a transformed offset term. Predictions may not be correct. Please apply transformation of offset term to the data before fitting the model and use 'offset=%s' in the model formula.\n", clean.term), "red")
+            warning(sprintf("Model uses a transformed offset term. Predictions may not be correct. Please apply transformation of offset term to the data before fitting the model and use 'offset(%s)' in the model formula.\n", clean.term), call. = FALSE)
+            olt <- clean.term
           }
-
-          check2 <- check2 & !check1
         }
       }
+      olt
     },
     error = function(x) { NULL },
     warning = function(x) { NULL },
@@ -131,6 +135,30 @@
   focal_terms <- c(focal_terms, constant_levels)
 
 
+  # check for monotonic terms and valid values. In case 'mo()' is used,
+  # and predictor is numeric, prettyfied values in the data grid are based
+  # on the range of the numeric variable, although only those values are allowed
+  # in the data grid that actually appear in the data
+
+  if (inherits(model, "brmsfit")) {
+    model_terms <- insight::find_terms(model, flatten = TRUE)
+    monotonics <- grepl("mo\\((.*)\\)", model_terms)
+    if (any(monotonics)) {
+      mo_terms <- gsub("mo\\((.*)\\)", "\\1", model_terms[monotonics])
+      invalid_levels <- unlist(lapply(mo_terms, function(mt) {
+        if (mt %in% names(focal_terms) && mt %in% colnames(model_frame)) {
+          !all(model_frame[[mt]] %in% focal_terms[[mt]])
+        } else {
+          FALSE
+        }
+      }))
+      if (any(invalid_levels)) {
+        stop(insight::format_message(sprintf("Variable(s) '%s' are used as monotonic effects, however, only values that are also present in the data are allowed for predictions. Consider converting variables used in 'mo()' into (ordered) factors before fitting the model.", paste0(mo_terms, collapse = ", "))), call. = FALSE)
+      }
+    }
+  }
+
+
   ## TODO check for other panelr models
 
   # get names of all predictor variable
@@ -175,9 +203,16 @@
 
   if (!inherits(model, "wbm")) {
 
-    if (sum(!(model_predictors %in% colnames(model_frame))) > 0 && !inherits(model, "brmsfit")) {
+    if (sum(!(model_predictors %in% colnames(model_frame))) > 0 && !inherits(model, c("brmsfit", "MCMCglmm"))) {
       # get terms from model directly
-      model_predictors <- attr(stats::terms(model), "term.labels", exact = TRUE)
+      model_predictors <- tryCatch(
+        {
+          attr(stats::terms(model), "term.labels", exact = TRUE)
+        },
+        error = function(e) {
+          NULL
+        }
+      )
     }
 
     # 2nd check
@@ -225,7 +260,7 @@
     constant_values <- lapply(model_predictors, function(x) {
       pred <- model_frame[[x]]
       if (!is.factor(pred) && !x %in% random_effect_terms) {
-        .typical_value(pred, fun = value_adjustment, weights = w, predictor = x, log_terms = .which_log_terms(model))
+        .typical_value(pred, fun = value_adjustment, weights = w, predictor = x, log_terms = .which_log_terms(model), emmeans.only = emmeans.only)
       }
     })
     names(constant_values) <- model_predictors
@@ -308,7 +343,7 @@
     focal_terms <- lapply(focal_term_names, function(x) {
       # check for consistent vector type: numeric
       if (is.numeric(model_frame[[x]]) && !is.numeric(focal_terms[[x]]))
-        return(sjlabelled::as_numeric(focal_terms[[x]]))
+        return(.factor_to_numeric(focal_terms[[x]]))
 
       # check for consistent vector type: factor
       if (is.factor(model_frame[[x]]) && !is.factor(focal_terms[[x]]))
@@ -339,7 +374,7 @@
 
 
   # create data frame with all unqiue combinations
-  dat <- as.data.frame(expand.grid(focal_terms))
+  dat <- as.data.frame(expand.grid(focal_terms, stringsAsFactors = TRUE))
 
 
   # we have to check type consistency. If user specified certain value
@@ -352,7 +387,7 @@
 
     # check for consistent vector type: numeric
     if (is.numeric(model_frame[[x]]) && !is.numeric(dat[[x]]))
-      return(sjlabelled::as_numeric(dat[[x]]))
+      return(.factor_to_numeric(dat[[x]]))
 
     # check for consistent vector type: factor
     if (is.factor(model_frame[[x]]) && !is.factor(dat[[x]]))
