@@ -1,5 +1,4 @@
-#' @importFrom insight link_inverse link_function
-.set_attributes_and_class <- function(data, model, t.title, x.title, y.title, l.title, legend.labels, x.axis.labels, model_info, constant.values = NULL, terms = NULL, original_terms = NULL, at_list = NULL, n.trials = NULL, prediction.interval = NULL, condition = NULL, ci.lvl = .95) {
+.set_attributes_and_class <- function(data, model, t.title, x.title, y.title, l.title, legend.labels, x.axis.labels, model_info, constant.values = NULL, terms = NULL, original_terms = NULL, at_list = NULL, n.trials = NULL, prediction.interval = NULL, condition = NULL, ci.lvl = .95, type = NULL) {
   # check correct labels
   if (!is.null(x.axis.labels) && length(x.axis.labels) != length(stats::na.omit(unique(data$x))))
     x.axis.labels <- as.vector(sort(stats::na.omit(unique(data$x))))
@@ -23,6 +22,8 @@
   attr(data, "prediction.interval") <- prediction.interval
   attr(data, "condition") <- condition
   attr(data, "ci.lvl") <- ci.lvl
+  attr(data, "type") <- type
+  attr(data, "response.name") <- insight::find_response(model)
 
   # remember fit family
   attr(data, "family") <- model_info$family
